@@ -40,6 +40,8 @@
 
 #define ALLOW_RULE(call) G_STMT_START { if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, SCMP_SYS(call), 0) < 0) goto out; } G_STMT_END
 
+#define ERROR_RULE(call, error) G_STMT_START { if (seccomp_rule_add (ctx, SCMP_ACT_ERRNO (error), SCMP_SYS(call), 0) < 0) goto out; } G_STMT_END
+
 gboolean
 tracker_seccomp_init (void)
 {
@@ -52,28 +54,46 @@ tracker_seccomp_init (void)
 	/* Memory management */
 	ALLOW_RULE (brk);
 	ALLOW_RULE (mmap);
+	ALLOW_RULE (mmap2);
 	ALLOW_RULE (munmap);
 	ALLOW_RULE (mremap);
 	ALLOW_RULE (mprotect);
 	ALLOW_RULE (madvise);
+	ERROR_RULE (mlock, EPERM);
+	ERROR_RULE (mlock2, EPERM);
+	ERROR_RULE (munlock, EPERM);
+	ERROR_RULE (mlockall, EPERM);
+	ERROR_RULE (munlockall, EPERM);
 	/* Process management */
 	ALLOW_RULE (exit_group);
 	ALLOW_RULE (getuid);
+	ALLOW_RULE (getuid32);
 	ALLOW_RULE (geteuid);
+	ALLOW_RULE (geteuid32);
 	ALLOW_RULE (getppid);
 	ALLOW_RULE (gettid);
+	ALLOW_RULE (getpid);
 	ALLOW_RULE (exit);
+	ALLOW_RULE (getrusage);
+	ALLOW_RULE (getrlimit);
 	/* Basic filesystem access */
 	ALLOW_RULE (fstat);
+	ALLOW_RULE (fstat64);
 	ALLOW_RULE (stat);
+	ALLOW_RULE (stat64);
 	ALLOW_RULE (statfs);
+	ALLOW_RULE (statfs64);
 	ALLOW_RULE (lstat);
+	ALLOW_RULE (lstat64);
 	ALLOW_RULE (access);
 	ALLOW_RULE (getdents);
+	ALLOW_RULE (getdents64);
 	ALLOW_RULE (readlink);
 	ALLOW_RULE (readlinkat);
 	ALLOW_RULE (utime);
+	ALLOW_RULE (time);
 	ALLOW_RULE (fsync);
+	ALLOW_RULE (umask);
 	/* Processes and threads */
 	ALLOW_RULE (clone);
 	ALLOW_RULE (futex);
@@ -87,6 +107,8 @@ tracker_seccomp_init (void)
 	ALLOW_RULE (poll);
 	ALLOW_RULE (ppoll);
 	ALLOW_RULE (fcntl);
+	ALLOW_RULE (fcntl64);
+	ALLOW_RULE (eventfd);
 	ALLOW_RULE (eventfd2);
 	ALLOW_RULE (pipe);
 	ALLOW_RULE (pipe2);
@@ -95,14 +117,19 @@ tracker_seccomp_init (void)
 	ALLOW_RULE (sysinfo);
 	ALLOW_RULE (prctl);
 	ALLOW_RULE (getrandom);
+	ALLOW_RULE (clock_gettime);
+	ALLOW_RULE (clock_getres);
+	ALLOW_RULE (gettimeofday);
 	/* Descriptors */
 	ALLOW_RULE (close);
 	ALLOW_RULE (read);
 	ALLOW_RULE (pread64);
 	ALLOW_RULE (lseek);
+	ALLOW_RULE (_llseek);
 	ALLOW_RULE (fadvise64);
 	ALLOW_RULE (write);
 	ALLOW_RULE (writev);
+	ALLOW_RULE (dup);
 	/* Needed by some GStreamer modules doing crazy stuff, less
 	 * scary thanks to the restriction below about sockets being
 	 * local.
@@ -153,6 +180,14 @@ tracker_seccomp_init (void)
 	                      SCMP_CMP(1, SCMP_CMP_MASKED_EQ, O_RDWR, O_RDWR)) < 0)
 		goto out;
 
+	/* Special requirements for dup2/dup3, no fiddling with stdin/out/err */
+	if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, SCMP_SYS(dup2), 1,
+	                      SCMP_CMP(1, SCMP_CMP_GT, 2)) < 0)
+		goto out;
+	if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, SCMP_SYS(dup3), 1,
+	                      SCMP_CMP(1, SCMP_CMP_GT, 2)) < 0)
+		goto out;
+
 	g_debug ("Loading seccomp rules.");
 
 	if (seccomp_load (ctx) >= 0)
